import React, {useContext} from "react";
import {ColumnKey, useColumn} from "./Column";
import {Smartable} from "./Smartable";

/**
 * Smartable cell definition.
 */
export interface CellDefinition<T = any>
{
	/**
	 * Cell data.
	 */
	data: T;

	/**
	 * Rendered cell element.
	 */
	element?: React.ReactElement;
}

/**
 * Default cell component.
 */
export function Cell({children}: React.PropsWithChildren<{}>)
{
	// Get cell data.
	const {data} = useCell();

	// Try to render cell data to string when no children given.
	return (
		<td>{children ?? String(data)}</td>
	);
}

/**
 * Table cell context data.
 */
export interface CellContextData<CK extends ColumnKey, T = any> extends CellDefinition<T>
{
}

const CellContext = React.createContext<CellContextData<ColumnKey>>(undefined);

/**
 * Hook to get current cell data.
 */
export function useCell<CK extends ColumnKey, T = any>(smartable?: Smartable<CK>): CellContextData<CK, T>
{
	return useContext(CellContext);
}

/**
 * Cell instance component.
 */
export function CellInstance<T>({cell}: {cell: CellDefinition<T>})
{
	// Get column data.
	const {column} = useColumn();

	return (
		<CellContext.Provider value={cell}>
			{ // Trying to render cell-specific element, then column-specific element, then default element.
				//TODO add row-specific element?
				cell.element ?? column.cellElement ?? <Cell />
			}
		</CellContext.Provider>
	)
}
