import React from "react";
import {Column, ColumnContext, ColumnHeading, ColumnKey, Columns} from "./Column";
import {SmartableProperties, useTable} from "./Smartable";
import {Async, Promisable} from "@kernelui/core";
import {RowDefinition, RowInstance} from "./Row";

/**
 * Smartable instance component properties.
 */
export interface InstanceProperties<CK extends ColumnKey> extends SmartableProperties<CK>
{
	/**
	 * Table columns.
	 */
	columns: Columns<CK>;
}

/**
 * Main component for a Smartable table.
 */
export function Instance<CK extends ColumnKey>({columns}: InstanceProperties<CK>)
{
	return (
		<table>
			<thead>
			<ColumnsHeadings columns={columns} />
			</thead>
			<tbody>
			<TableBody />
			</tbody>
		</table>
	);
}

/**
 * Columns headings of a Smartable table.
 */
export function ColumnsHeadings<CK extends ColumnKey>({columns}: {columns: Columns<CK>})
{
	return (
		<tr className={"headings"}>
			{ // Showing title of each column.
				Object.entries(columns).map(([key, column]) => (
					<ColumnContext.Provider key={key} value={{
						key: key,
						column: column as Column,
					}}>
						<ColumnHeading/>
					</ColumnContext.Provider>
				))
			}
		</tr>
	);
}

export function TableBody<CK extends ColumnKey>()
{
	// Get data from table.
	const {data} = useTable();

	return (
		<Async<(Promisable<RowDefinition<CK>>)[]> promise={data}>
			{(rowsData) => (
				// Rendering defined rows.
				<>
					{ // Rendering each row.
						rowsData.map((rowData, index) => (
							// Rendering current row from its definition.
							<Async<RowDefinition<CK>> key={index} promise={rowData}>
								{(rowDefinition) => (
									<RowInstance row={rowDefinition} />
								)}
							</Async>
						))
					}
				</>
			)}
		</Async>
	);
}
